import os
import unittest
import tempfile

from twisted.words.protocols.jabber.jid import InvalidFormat

from gadget.roster import GadgetRoster

class RosterTest(unittest.TestCase):
    def setUp(self):
        self.path = tempfile.mkstemp('.roster')[1]

    def tearDown(self):
        os.unlink(self.path)

    def write_roster_file(self, jids):
        file(self.path, 'w').write(''.join(jid + '\n' for jid in jids))

    def expect(self, *jids):
        self.assertEquals(set(jids), set(GadgetRoster(self.path)))

    def test_roster(self):
        # Create a file with some JIDs.
        self.write_roster_file(['alice@foo.org', 'bob@baz.net'])
        self.expect('alice@foo.org', 'bob@baz.net')

        # Add a JID.
        GadgetRoster(self.path).add('charles@localhost')
        self.expect('alice@foo.org', 'bob@baz.net', 'charles@localhost')
        assert 'alice@foo.org' in GadgetRoster(self.path)

        # Re-add the same JID.
        GadgetRoster(self.path).add('charles@localhost')
        self.expect('alice@foo.org', 'bob@baz.net', 'charles@localhost')

        # Remove Alice.
        GadgetRoster(self.path).remove('alice@foo.org')
        self.expect('bob@baz.net', 'charles@localhost')
        assert 'alice@foo.org' not in GadgetRoster(self.path)

        # Remove a JID that isn't there.
        GadgetRoster(self.path).remove('lucien@foo.org')
        self.expect('bob@baz.net', 'charles@localhost')

    def test_roster_no_file(self):
        roster = GadgetRoster("this roster doesn't exist")
        self.assertEquals(set(), set(roster))

    def test_bad_roster(self):
        # Invalid JIDs raise an exception.
        self.write_roster_file(['okok@foo.org', 'bad@bad@foo.org'])
        self.assertRaises(InvalidFormat, GadgetRoster, self.path)

        # JIDs with resources are ignored.
        self.write_roster_file(['okok@foo.org', 'jid@with/resource'])
        self.expect('okok@foo.org')

